source('supp_utils.R')

data_filename_list <- c('mouse_retina_drop-seq.rds', 'mouse_sie_10x.rds',
                        'human_blood.rds')
cell_type_sel_list <- matrix(c('bipolar', 'cone', 'rod',
                               'Stem', 'Goblet', 'Tuft',
                               'DC1', 'DC2', 'Mono2'),
                             byrow = FALSE, nrow = 3)
protocol_list <- c('drop-seq', '10x', 'SMART-Seq2')

for(iter1 in 1:length(data_filename_list))
  for(iter2 in 1:nrow(cell_type_sel_list)){
    data_filename <- data_filename_list[iter1]
    cell_type_sel <- cell_type_sel_list[iter2, iter1]
    output_suffix <- paste0(protocol_list[iter1], '_', cell_type_sel)
    
    print(output_suffix)

    temp <- get_data_single_cell_type(data_filename, cell_type_sel)
    data_mat <- cbind(temp$testcount, temp$realcount)
    cat('cell number =', ncol(data_mat), '\n')
    
    copula_model <- readRDS(file = paste0('copula_model_', output_suffix, '.rds'))
    
    min_value <- 1e-5
    print('gene group 1:')
    n_ZINB <- sum(copula_model$marginal_param1[, 1] > min_value &
                    !is.infinite(copula_model$marginal_param1[, 2]))
    n_ZIP <- sum(copula_model$marginal_param1[, 1] > min_value &
                   is.infinite(copula_model$marginal_param1[, 2]))
    n_NB <- sum(copula_model$marginal_param1[, 1] <= min_value &
                  !is.infinite(copula_model$marginal_param1[, 2]))
    n_Poisson <- sum(copula_model$marginal_param1[, 1] <= min_value &
                       is.infinite(copula_model$marginal_param1[, 2]))
    cat('# gene =', nrow(copula_model$marginal_param1), '\n')
    cat('# ZINB =', n_ZINB, '\n')
    cat('# ZIP =', n_ZIP, '\n')
    cat('# NB =', n_NB, '\n')
    cat('# Poisson =', n_Poisson, '\n')
    cat('# ZI =', n_ZINB + n_ZIP, '\n')
    cat('# non-ZI =', n_NB + n_Poisson, '\n')
    
    print('gene group 2:')
    n_ZINB <- sum(copula_model$marginal_param2[, 1] > min_value &
                    !is.infinite(copula_model$marginal_param2[, 2]))
    n_ZIP <- sum(copula_model$marginal_param2[, 1] > min_value &
                   is.infinite(copula_model$marginal_param2[, 2]))
    n_NB <- sum(copula_model$marginal_param2[, 1] <= min_value &
                  !is.infinite(copula_model$marginal_param2[, 2]))
    n_Poisson <- sum(copula_model$marginal_param2[, 1] <= min_value &
                       is.infinite(copula_model$marginal_param2[, 2]))
    cat('# gene =', nrow(copula_model$marginal_param2), '\n')
    cat('# ZINB =', n_ZINB, '\n')
    cat('# ZIP =', n_ZIP, '\n')
    cat('# NB =', n_NB, '\n')
    cat('# Poisson =', n_Poisson, '\n')
    cat('# ZI =', n_ZINB + n_ZIP, '\n')
    cat('# non-ZI =', n_NB + n_Poisson, '\n\n')
    
  }



for(iter1 in 1:length(data_filename_list))
  for(iter2 in 1:nrow(cell_type_sel_list)){
    data_filename <- data_filename_list[iter1]
    cell_type_sel <- cell_type_sel_list[iter2, iter1]
    output_suffix <- paste0(protocol_list[iter1], '_', cell_type_sel)
    
    print(output_suffix)
    
    temp <- get_data_single_cell_type(data_filename, cell_type_sel)
    data_mat <- cbind(temp$testcount, temp$realcount)
    cat('cell number =', ncol(data_mat), '\n')
    
    copula_model <- readRDS(file = paste0('copula_model_', output_suffix, '.rds'))
    
    min_value <- 1e-5
    n_ZI1 <- sum(copula_model$marginal_param1[, 1] > min_value)
    n_non_ZI1 <- sum(copula_model$marginal_param1[, 1] <= min_value)
    n_ZI2 <- sum(copula_model$marginal_param2[, 1] > min_value)
    n_non_ZI2 <- sum(copula_model$marginal_param2[, 1] <= min_value)
    cat('# gene fitted =', nrow(copula_model$marginal_param1) +
          nrow(copula_model$marginal_param2), '\n')
    cat('# ZI =', n_ZI1 + n_ZI2, '\n')
    cat('# non-ZI =', n_non_ZI1 + n_non_ZI2, '\n\n')
    
  }



