library(scDesign2)
# source('supp_utils.R')
library(SingleCellExperiment)

data_filename_list <- c('mouse_retina_drop-seq.rds', 'mouse_sie_10x.rds',
                        'human_blood.rds')
cell_type_sel_list <- matrix(c('bipolar', 'rod', 'cone',
                               'Stem', 'Goblet', 'Tuft',
                               'DC1', 'DC2', 'Mono2'),
                             byrow = FALSE, nrow = 3)
protocol_list <- c('drop-seq', '10x', 'SMART-Seq2')

data_filename_list <- c('pbmc2_hca_drop-seq.rds', 'pbmc2_hca_10x.rds',
                        'pbmc2_hca_SMART-Seq2.rds')
# cell_type_sel <- c('B cell', 'CD14+ monocyte',
#                    'Natural killer cell', 'CD4+ T cell',
#                    'Cytotoxic T cell')
cell_type_sel_list <- matrix(rep(c('B cell', 'CD14+ monocyte',
                                   'Natural killer cell', 'CD4+ T cell',
                                   'Cytotoxic T cell'), length(data_filename_list)),
                             byrow = FALSE, ncol = length(data_filename_list))
cell_type_sel_list
protocol_list <- c('drop-seq', '10x', 'SMART-Seq2')

for(iter1 in 1:length(data_filename_list)){
  # gene names
  data_filename <- data_filename_list[iter1]
  data_mat <- readRDS(data_filename)
  gene_name <- rownames(data_mat)
  
  # which genes are de & modified params
  load(file = paste0('data_param_new_', protocol_list[iter1], '_de.rda'))
  de_vec <- rep(0, nrow(data_mat))
  de_vec[gene_idx_remain] <- 1
  
  # data - copula
  data_sim_copula <- lapply(1:nrow(cell_type_sel_list), function(iter2){
    cell_type_sel <- cell_type_sel_list[iter2, iter1]
    output_suffix <- paste0(protocol_list[iter1], '_', cell_type_sel)
    if(iter2 > 3){
      return(readRDS(file = paste0('data_sim_copula_', output_suffix, '_de.rds')))
    }else{
      return(readRDS(file = paste0('data_sim_copula_', output_suffix, '.rds')))
    }
  })
  # data - nozi
  data_sim_nozi <- lapply(1:nrow(cell_type_sel_list), function(iter2){
    cell_type_sel <- cell_type_sel_list[iter2, iter1]
    output_suffix <- paste0(protocol_list[iter1], '_', cell_type_sel)
    if(iter2 > 3){
      return(readRDS(file = paste0('data_sim_nozi_', output_suffix, '_de.rds')))
    }else{
      return(readRDS(file = paste0('data_sim_nozi_', output_suffix, '.rds')))
    }
  })
  # copula models for non-de cell types
  data_param_new_contd <- lapply(1:3, function(iter2){ # 3:nrow(cell_type_sel_list)
    cell_type_sel <- cell_type_sel_list[iter2, iter1]
    output_suffix <- paste0(protocol_list[iter1], '_', cell_type_sel)
    copula_model <- readRDS(file = paste0('copula_model_', output_suffix, '.rds'))
    return(list(copula_model = copula_model[[1]], n_cell = ncol(data_sim_copula[[iter2]])))
  })
  
  data_param_new <- c(data_param_new, data_param_new_contd)
  
  param_mat <- matrix(0.0, nrow = length(gene_name), ncol = 3*length(data_param_new))
  param_mat[, (1:length(data_param_new))*3-1] <- Inf
  for(iter2 in 1:length(data_param_new)){
    param_mat[data_param_new[[iter2]]$copula_model$gene_sel1, (3*iter2-2):(3*iter2)] <-
      data_param_new[[iter2]]$copula_model$marginal_param1
    param_mat[data_param_new[[iter2]]$copula_model$gene_sel2, (3*iter2-2):(3*iter2)] <-
      data_param_new[[iter2]]$copula_model$marginal_param2
  }
  param_mat <- cbind(param_mat, de_vec)
  colnames(param_mat) <- c(as.character(outer(c('param_zi_', 'param_dispersion_', 'param_mu_'),
                                            cell_type_sel_list[, iter1], paste0)),
                           paste0('de_', cell_type_sel_list[1, iter1], '_',
                                  cell_type_sel_list[2, iter1]))
  row_data <- data.frame(param_mat)
  row_data$gene <- gene_name
  row_data <- DataFrame(row_data)
  # names(gene_name) <- NULL
  # row_data@rownames <- gene_name
  
  data_sim_copula <- Reduce(cbind, data_sim_copula)
  data_sim_nozi <- Reduce(cbind, data_sim_nozi)

  col_data <- DataFrame(cell_type = unlist(lapply(1:nrow(cell_type_sel_list), function(iter2){
    cell_type_sel <- cell_type_sel_list[iter2, iter1]
    rep(cell_type_sel, data_param_new[[iter2]]$n_cell)
  })), cell = paste0('cell', 1:ncol(data_sim_copula)))
  col_data@rownames <- paste0('cell', 1:ncol(data_sim_copula))
  
  rownames(data_sim_copula) <- gene_name
  colnames(data_sim_copula) <- paste0('cell', 1:ncol(data_sim_copula))
  rownames(data_sim_nozi) <- gene_name
  colnames(data_sim_nozi) <- paste0('cell', 1:ncol(data_sim_nozi))
  sce <- SingleCellExperiment(list(counts_original = data_sim_copula,
                                   counts_nozi = data_sim_nozi))
  colData(sce) <- col_data
  rowData(sce) <- row_data
  # rownames(rowData(sce)) <- gene_name
  saveRDS(sce, paste0('sce_', protocol_list[iter1], '_pbmc2_hca.rds'))
}



