source('gex_lib.R')

param_true <- rbind(c(1/3,1/3,200,0.4),
                    c(3,1/3,200,0.4),
                    c(1/3,3,200,0.4))

n_model <- nrow(param_true)

# process data generation ----------------------------------------
set.seed(1)
trim_factor <- 0.9
process_data <- lapply(1:n_model, function(iter_test){
  test_test <- two_state_gex_model(0, 0, param_true[iter_test, 1],
                                   param_true[iter_test, 2],
                                   param_true[iter_test, 3])
  realization <- test_test$realization
  t_max <- test_test$t_max
  
  n_reaction <- length(realization)
  realization <- matrix(unlist(realization), nrow = n_reaction, byrow = T)
  time_axis <- cumsum(realization[, 1])
  
  sim_sel <- which(time_axis > t_max * trim_factor)
  time_axis <- cumsum(realization[sim_sel, 1])
  realization <- realization[sim_sel, ]
  
  return(list(time_axis = time_axis, realization = realization))
})

names(process_data) <- as.character(1:n_model)
saveRDS(process_data, 'gex_process_data.rds')
# plot(time_axis, realization[,2], type = 's')


# distribution data generation ----------------------------------------
set.seed(1)
n_sample <- 1000
dist_data <- lapply(1:n_model, function(iter_test){
  random_sample <- sapply(1:n_sample, function(x){
    test_test <- two_state_gex_model(0, 0, param_true[iter_test, 1],
                                     param_true[iter_test, 2],
                                     param_true[iter_test, 3])
    realization <- test_test$realization
    return(tail(realization, 1)[[1]][[2]])
  })
  return(random_sample)
})
names(dist_data) <- as.character(1:n_model)
saveRDS(dist_data, 'gex_dist_data.rds')
# hist(random_sample, breaks = 20)







