library(ggpubr)
library(ggplot2)
# install.packages("ZIM")
library(ZIM)
library(gamlss.dist)

dist_plot <- function(mu = 5, disp = 2, zp = 0.2, samp_size = 1000){
  df <- data.frame(x = rpois(samp_size, mu))
  p1 <- ggplot(data = df, aes(x = x)) + geom_density(fill="#636363", color = "#636363") +
    xlab(paste0("zp = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""))
    # geom_text(label = paste0("zp = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""), x = 10, y = 0.1,
    #            size = 10)
  
  mu_zi = mu / (1-zp)
  df <- data.frame(x = rZIP(samp_size, mu_zi, zp))
  p2 <- ggplot(data = df, aes(x = x)) + geom_density(fill="#636363", color = "#636363") +
    xlab(paste0("zp = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""))
  
  df <- data.frame(x = rnegbin(samp_size, mu, disp))
  p3 <- ggplot(data = df, aes(x = x)) + geom_density(fill="#636363", color = "#636363") +
    xlab(paste0("zp = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""))
  
  df <- data.frame(x = rzinb(samp_size, disp, mu, zp))
  p4 <- ggplot(data = df, aes(x = x)) + geom_density(fill="#636363", color = "#636363") +
    xlab(paste0("zp = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""))
  
  figure <- ggarrange(p1, p2, p3, p4,
                      labels = c("A", "B", "C", "D"),
                      ncol = 2, nrow = 2)
  return(figure)
}


dist_plot_2 <- function(mu = 5, disp = 2, zp = 0.2, samp_size = 1000){
  df <- data.frame(x = rpois(samp_size, mu))
  p1 <- ggplot(data = df, aes(x = x)) + geom_histogram(fill="#636363", color = "#636363", binwidth = 0.5) +
    xlab(paste0("zero proportion = ", sum(df$x == 0) / samp_size * 100, "%", collapse = "")) +
    aes(y=stat(count)/sum(stat(count))) + theme_bw() + ylab("proportion") + xlim(-0.5, 15) + ylim(0, 0.7) +
    ggtitle("Poisson")
  # geom_text(label = paste0("zp = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""), x = 10, y = 0.1,
  #            size = 10)
  
  # hist(rZIP(samp_size, mu_zi, zp), probability = TRUE)
  
  mu_zi = mu / (1-zp)
  df <- data.frame(x = rZIP(samp_size, mu_zi, zp))
  p2 <- ggplot(data = df, aes(x = x)) + geom_histogram(fill="#636363", color = "#636363", binwidth = 0.5) +
    xlab(paste0("zero proportion = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""))+
    aes(y=stat(count)/sum(stat(count))) + theme_bw()+ ylab("proportion")  + xlim(-0.5, 15) + ylim(0, 0.7) +
    ggtitle("Zero-inflated Poisson")
  
  df <- data.frame(x = rnegbin(samp_size, mu, disp))
  p3 <- ggplot(data = df, aes(x = x)) + geom_histogram(fill="#636363", color = "#636363", binwidth = 0.5) +
    xlab(paste0("zero proportion = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""))+
    aes(y=stat(count)/sum(stat(count))) + theme_bw()+ ylab("proportion")  + xlim(-0.5, 15) + ylim(0, 0.7) +
    ggtitle("Negative Binomial")
  
  df <- data.frame(x = rzinb(samp_size, disp, mu_zi, zp))
  p4 <- ggplot(data = df, aes(x = x)) + geom_histogram(fill="#636363", color = "#636363", binwidth = 0.5) +
    xlab(paste0("zero proportion = ", sum(df$x == 0) / samp_size * 100, "%", collapse = ""))+
    aes(y=stat(count)/sum(stat(count))) + theme_bw()+ ylab("proportion")  + xlim(-0.5, 15) + ylim(0, 0.7) +
    ggtitle("Zero-inflated Negative Binomial")
  
  figure <- ggarrange(p1, p2, p3, p4,
                      labels = c("A", "B", "C", "D"),
                      ncol = 2, nrow = 2)
  return(figure)
}
