library(ggplot2); theme_set(theme_bw());
# library(ggpattern)
library(plyr) # mapvalues
library(dplyr)
library(gridExtra) # plot layout
library(cowplot) # draw_plot_label, ggsave2
library(ggpubr) # as_ggplot
# library(reshape2)
# library(RColorBrewer)

data_summary_list <- readRDS('data_summary_list.rds')
# gene_sel_dat_list <- readRDS('gene_sel_dat_list.rds')
gene_sel_dat_list <- readRDS('gene_sel_dat_list_B cell.rds')

data_summary_dat <- Reduce(rbind, lapply(1:length(data_summary_list), function(iter1){
  x <- data_summary_list[[iter1]]
  x_mark <- names(data_summary_list)[[iter1]]
  protocol <- unlist(strsplit(x_mark, '_'))[1]
  cell_type <- unlist(strsplit(x_mark, '_'))[2]
  Reduce(rbind, lapply(1:length(x), function(iter2){
    data.frame(value = x[[iter2]], measure = names(x)[[iter2]],
               protocol = protocol, cell_type = cell_type,
               group = x_mark,
               stringsAsFactors = FALSE)
  }))
}))

# data_summary_dat$value[data_summary_dat$measure == 'cell_libsize'] <-
#   log(data_summary_dat$value[data_summary_dat$measure == 'cell_libsize'])

# data_summary_dat$cell_type <- factor(data_summary_dat$cell_type,
#                                      levels = c('rod', 'bipolar', 'cone',
#                                                 'Stem', 'Goblet', 'Tuft',
#                                                 'DC1', 'Mono2', 'DC2'))
data_summary_dat$protocol <- mapvalues(data_summary_dat$protocol,
                                       from = c('drop-seq', '10x', 'SMART-Seq2'),
                                       to = c('Drop-seq', '10x Genomics', 'Smart-seq2'))
model_param_dat <- subset(data_summary_dat, measure %in% c('n_ZI', 'n_non_ZI')) # , 'n_nofit'
model_param_dat$measure <- mapvalues(model_param_dat$measure,
                                     from = c('n_ZI', 'n_non_ZI'), #, 'n_nofit'
                                     to = c('ZI', 'non-ZI')) # , 'no fit'
model_param_dat$measure <- factor(model_param_dat$measure, levels = c('non-ZI', 'ZI')) # 'no fit', 
model_param_dat <- group_by(model_param_dat, group)
model_param_dat <- mutate(model_param_dat, prop = value / sum(value))
# model_param_dat$group2 <- apply(model_param_dat[, c('cell_type', 'protocol')], 1, function(x){
#   paste0(x[1], '_', x[2])
# })
model_param_dat
model_param_dat_fake_non_ZI <- subset(model_param_dat, subset = measure == 'non-ZI')
model_param_dat_fake_non_ZI$prop <- 1.0
model_param_dat_ZI <- subset(model_param_dat, subset = measure == 'ZI')

tag_sel <- c('drop-seq_CD4+ T cell', '10x_CD4+ T cell', 'SMART-Seq2_CD4+ T cell')
tag_sel <- c('drop-seq_B cell', '10x_B cell', 'SMART-Seq2_B cell')
dzinb <- function(x, prob0, size, mu){
  prob0 * (x == 0) + (1 - prob0) * dnbinom(x, size = size, mu = mu)
}

gene_sel_dat <- Reduce(rbind, lapply(tag_sel, function(x_mark){
  x <- gene_sel_dat_list[[x_mark]][['scatter_dat']]
  protocol <- unlist(strsplit(x_mark, '_'))[1]
  cell_type <- unlist(strsplit(x_mark, '_'))[2]
  gene_param_mat <- gene_sel_dat_list[[x_mark]][['param_dat']]
  print(x_mark)
  
  Reduce(rbind, lapply(1:nrow(x), function(iter2){
    summ <- table(x[iter2, ])
    gene_param <- gene_param_mat[iter2, ]
    
    x_pos <- seq(from = min(as.integer(names(summ))),
                 to = max(as.integer(names(summ))))
    x_pos
    real_prop <- rep(0.0, length(x_pos))
    names(real_prop) <- as.character(x_pos)
    real_prop[names(summ)] <- as.numeric(summ)/ncol(x)
    fitted_prop <- dzinb(x_pos,
                         gene_param[1], gene_param[2], gene_param[3])
    
    data.frame(x = rep(x_pos, 2),
               value = c(real_prop, fitted_prop),
               prop_label = c(rep('real data', length(x_pos)),
                              rep('fitted model', length(x_pos))),
               model_type = if(iter2 <=2 ) 'ZI' else 'non-ZI',
               gene_name = rownames(x)[iter2],
               gene_id = as.character(iter2),
               protocol = protocol, cell_type = cell_type,
               group = x_mark,
               stringsAsFactors = FALSE)
  }))
}))

gene_sel_dat_hist_real <- Reduce(rbind, lapply(tag_sel, function(x_mark){
  x <- gene_sel_dat_list[[x_mark]][['scatter_dat']]
  protocol <- unlist(strsplit(x_mark, '_'))[1]
  cell_type <- unlist(strsplit(x_mark, '_'))[2]
  gene_param_mat <- gene_sel_dat_list[[x_mark]][['param_dat']]
  print(x_mark)
  
  Reduce(rbind, lapply(1:nrow(x), function(iter2){
    data.frame(x = x[iter2, ],
               gene_name = rownames(x)[iter2],
               gene_id = as.character(iter2),
               protocol = protocol, cell_type = cell_type,
               group = x_mark,
               stringsAsFactors = FALSE)
  }))
}))

gene_sel_dat_hist_breaks <- lapply(tag_sel, function(x_mark){
  x <- gene_sel_dat_list[[x_mark]][['scatter_dat']]
  protocol <- unlist(strsplit(x_mark, '_'))[1]
  cell_type <- unlist(strsplit(x_mark, '_'))[2]
  gene_param_mat <- gene_sel_dat_list[[x_mark]][['param_dat']]
  print(x_mark)
  
  breaks_list <- lapply(1:nrow(x), function(iter2){
    count_vec <- x[iter2, ]
    seq(min(count_vec)-0.5, max(count_vec)+0.5, length.out = 30)
  })
  names(breaks_list) <- rownames(x)
  breaks_list
})
names(gene_sel_dat_hist_breaks) <- tag_sel

gene_sel_dat_bin_fitted_prop <- Reduce(rbind, lapply(tag_sel, function(x_mark){
  x <- gene_sel_dat_list[[x_mark]][['scatter_dat']]
  protocol <- unlist(strsplit(x_mark, '_'))[1]
  cell_type <- unlist(strsplit(x_mark, '_'))[2]
  gene_param_mat <- gene_sel_dat_list[[x_mark]][['param_dat']]
  print(x_mark)
  
  Reduce(rbind, lapply(1:nrow(x), function(iter2){
    gene_param <- gene_param_mat[iter2, ]
    
    breaks <- gene_sel_dat_hist_breaks[[x_mark]][[iter2]]
    # print(breaks)
    n_breaks <- length(breaks)
    intervals <- cbind(breaks[1:(n_breaks-1)], breaks[2:n_breaks])
    intervals <- intervals[ceiling(intervals[, 1]) <= floor(intervals[, 2]), ]
    
    fitted_prop <- apply(intervals, 1, function(interval){
      llim <- ceiling(interval[1])
      ulim <- floor(interval[2])
      sum(dzinb(llim:ulim, gene_param[1], gene_param[2], gene_param[3]))
    })

    data.frame(x = apply(intervals, 1, mean),
               fitted_prop = fitted_prop,
               gene_name = rownames(x)[iter2],
               gene_id = as.character(iter2),
               protocol = protocol, cell_type = cell_type,
               group = x_mark,
               stringsAsFactors = FALSE)
  }))
}))

data_temp <- data.frame(x = c(rep(0, 10), rep(1, 5), rep(2, 2), 3))
ggplot(data_temp, aes(x = x)) + geom_histogram(breaks = seq(-0.5,3.5,1))

gene_name_ordered <- unlist(lapply(tag_sel, function(x_mark){
  x <- gene_sel_dat_list[[x_mark]][['scatter_dat']]
  rownames(x)
}))
gene_name_ordered <- c('EEF1A1', 'ACTB', 'CD79B', 'LCP1')

gene_sel_dat$gene_name <- factor(gene_sel_dat$gene_name, levels = gene_name_ordered)
gene_sel_dat_hist_real$gene_name <- factor(gene_sel_dat_hist_real$gene_name, levels = gene_name_ordered)
gene_sel_dat_bin_fitted_prop$gene_name <-
  factor(gene_sel_dat_bin_fitted_prop$gene_name, levels = gene_name_ordered)
gene_sel_dat_hist_breaks <- lapply(1:length(gene_sel_dat_hist_breaks), function(iter){
  gene_sel_dat_hist_breaks[[iter]][gene_name_ordered]
})
gene_sel_dat$prop_label <- factor(gene_sel_dat$prop_label,
                                  levels = c('real data', 'fitted model'))
tag_sel_new <- c('Drop-seq\n CD4+ T cell', '10x Genomics\n CD4+ T cell',
                 'Smart-seq2\n  CD4+ T cell')
tag_sel_new <- c('10x Genomics\n     B cell', 'Drop-seq\n   B cell',
                 'Smart-seq2\n    B cell')
gene_sel_dat$group <- mapvalues(gene_sel_dat$group, from = tag_sel, tag_sel_new)
gene_sel_dat_hist_real$group <- mapvalues(gene_sel_dat_hist_real$group, from = tag_sel, tag_sel_new)
gene_sel_dat_bin_fitted_prop$group <-
  mapvalues(gene_sel_dat_bin_fitted_prop$group, from = tag_sel, tag_sel_new)
names(gene_sel_dat_hist_breaks) <- tag_sel_new

# plotting ---------------------------------------------------------------------------
cell_libsize_plot <- ggplot(mapping = aes(x = cell_type, y = value, color = protocol)) +
  geom_violin(data = subset(data_summary_dat, measure == 'cell_libsize'),
              scale = 'width') +
  scale_y_continuous(trans = 'log10') +
  xlab('') + ylab('cell library size') +
  theme(legend.position = 'none',
        axis.text.x = element_text(angle = 45, hjust = 1))

n_gene_plot <- ggplot(mapping = aes(x = cell_type, y = value, color = protocol)) +
  geom_violin(data = subset(data_summary_dat, measure == 'n_gene_detected'),
              scale = 'width') +
  xlab('') + ylab('number of genes\ndetected') +
  theme(legend.position = 'none',
        axis.text.x = element_text(angle = 45, hjust = 1))

n_cell_plot <- ggplot(mapping = aes(x = cell_type, y = value, fill = protocol)) +
  geom_bar(data = subset(data_summary_dat, measure == 'n_cell'),
           position="dodge", stat='identity') +
  xlab('') + ylab('number of cells') +
  theme(legend.position = 'none',
        axis.text.x = element_text(angle = 45, hjust = 1))

col_model_type <- c('#C0C0C0', '#696969', 'black')
names(col_model_type) <- c('no fit', 'non-ZI', 'ZI')
col_model_type <- c('#C0C0C0', 'black')
names(col_model_type) <- c('non-ZI', 'ZI')
# model_param_dat_ZI$ZI <- model_param_dat_ZI$protocol
# model_param_dat_ZI$protocol <- 'ZI'
# model_param_dat_for_plot <- rbind(model_param_dat_fake_non_ZI,
#                                   model_param_dat_ZI)
# model_param_dat_for_plot$protocol <- as.factor(model_param_dat_for_plot$protocol)
model_param_dat$protocol[model_param_dat$measure == 'ZI'] <- 'ZI'
# model_param_dat
model_param_dat$group <- as.factor(model_param_dat$group)
# model_param_dat_for_plot
# levels(model_param_dat$group)
model_param_dat$group <- factor(model_param_dat$group,
                                levels = as.vector(t(matrix(levels(model_param_dat$group),
                                                            nrow = 5, ncol = 3))))
model_param_dat$cell_type <- as.factor(model_param_dat$cell_type)
x_axis_labels <- rep('', 5 * 3)
x_axis_labels[(1:(5*3)) %% 3 == 2] <- levels(model_param_dat$cell_type)
model_param_plot <- ggplot(model_param_dat,
                           mapping = aes(x = group, y = prop, fill = protocol)) +
  geom_col(position = 'fill') + #width=0.8 width=0.7,
  xlab('') + ylab('proportion') +
  scale_fill_manual(values = c('#F8766D', '#00BA38', '#619CFF', 'black')) +
  scale_x_discrete(breaks = paste0('drop-seq_', levels(model_param_dat$cell_type)),
                   labels = levels(model_param_dat$cell_type)) +
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        axis.text.x = element_text(angle = 45, hjust = 1),
        # legend.spacing.y = unit(0.2, "in"),
        legend.position = 'none',
        legend.key.size = unit(0.1, "in"))
# model_param_plot

get_legend<-function(myggplot){
  tmp <- ggplot_gtable(ggplot_build(myggplot))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)
}
protocol_aux <- ggplot(model_param_dat_fake_non_ZI,
                       mapping = aes(x = group, y = prop, fill = protocol)) +
  geom_col() +
  theme(# legend.text=element_text(size=10),
        legend.key.size = unit(0.1, "in"))
model_type_aux <- ggplot(model_param_dat_ZI,
                         mapping = aes(x = group, y = prop, fill = measure)) +
  geom_col() +
  scale_fill_manual(name = 'model type    ',
                    values = 'black') +
  theme(legend.key.size = unit(0.1, "in"))
  
legend_protocol <- get_legend(protocol_aux)
legend_model_type <- get_legend(model_type_aux)
df <- data.frame()
plot_empty <- ggplot(df) + geom_point() + theme(panel.border=element_blank())

# gene histogram --------------------------------------------------------------------
gene_prop_plot <- lapply(tag_sel_new, function(x_mark){
  ggplot(data = subset(gene_sel_dat, subset = group == x_mark),
         aes(x = x, color = prop_label)) + # , lty = model_type
    geom_line(aes(y = value)) +
    facet_wrap(~gene_name, scales = 'free', nrow = 1) +
    labs(x='', y='proportion') +
    scale_color_discrete(name = x_mark) #+
  # guides(lty=guide_legend(title='model type'))
})

gene_hist_plot <- lapply(tag_sel_new, function(x_mark){
  d <- subset(gene_sel_dat_hist_real, subset = group == x_mark)
  breaks <- gene_sel_dat_hist_breaks[[x_mark]]
  ggplot(mapping = aes(x = x)) + # , lty = model_type
    mapply(function(d, b) {
      geom_histogram(data=d, breaks=b, fill = '#F8766D',
                     aes(y = stat(count / sum(count)), color = 'a'))}, 
      split(d, d$gene_name), breaks) +
    # geom_histogram(aes(y = stat(count / sum(count))), stat = 'bin') +
    geom_line(data = subset(gene_sel_dat_bin_fitted_prop, group == x_mark),
              aes(y = fitted_prop, color = 'b'), lwd = 1.1) +
    facet_wrap(~gene_name, scales = 'free', nrow = 1) +
    labs(x='', y='proportion') +
    scale_color_manual(name = x_mark,
                       values = c('a' = '#F8766D', 'b' = '#00BFC4'),
                       labels = c('real data', 'fitted model')) +
    guides(color = guide_legend(override.aes = list(fill = "white"))) +
    theme(strip.text = element_text(face = 'italic'))
    # scale_fill_manual(values = c('#F8766D', 'white'))
    # scale_color_discrete(name = x_mark) #+
  # guides(lty=guide_legend(title='model type'))
})
gene_hist_plot[[3]]

df <- data.frame()
annotation_plot1 <- ggplot(df) + geom_point() + xlim(0, 5) + ylim(0, 1) +
  annotate('text', x = 1.05, y = 0.5, label = 'no ZI', size = 5) +
  annotate('text', x = 3.4, y = 0.5, label = 'no ZI', size = 5) +
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border=element_blank(),
        axis.line=element_blank(),
        axis.title=element_blank(),
        axis.text=element_blank(),
        axis.ticks=element_blank())
annotation_plot1

annotation_plot2 <- ggplot(df) + geom_point() + xlim(0, 5) + ylim(0, 1) +
  annotate('text', x = 1.05, y = 0.5, label = 'no ZI', size = 5) +
  annotate('text', x = 3.4, y = 0.5, label = 'ZI', size = 5) +
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border=element_blank(),
        axis.line=element_blank(),
        axis.title=element_blank(),
        axis.text=element_blank(),
        axis.ticks=element_blank())
# annotation_plot1

# final plot ---------------------------------------------------------------------------
gt <- arrangeGrob(arrangeGrob(cell_libsize_plot, n_cell_plot, n_gene_plot, 
                              model_param_plot,
                              arrangeGrob(legend_protocol, legend_model_type,
                                          heights = c(1,2)),
                              widths = c(1,1,1,1,0.5)),
                  annotation_plot1,
                  gene_prop_plot[[1]],
                  gene_prop_plot[[2]],
                  annotation_plot2,
                  gene_hist_plot[[3]], heights = c(1.3,0.2,1,1,0.2,1))
final_plot <- as_ggplot(gt)

ggsave2("data_summary_visualization_B cell.png", device = "png",
        plot = final_plot, width = 9.5, height = 10, units = "in", dpi = 300)

gt <- arrangeGrob(arrangeGrob(cell_libsize_plot, n_cell_plot, n_gene_plot, 
                              model_param_plot,
                              arrangeGrob(legend_protocol, legend_model_type,
                                          plot_empty,
                                          heights = c(1.3,0.4,1)),
                              widths = c(1,1,1,1,0.5)),
                  annotation_plot1,
                  gene_hist_plot[[1]],
                  gene_hist_plot[[2]],
                  annotation_plot2,
                  gene_hist_plot[[3]], heights = c(1.3,0.2,1,1,0.2,1))
final_plot <- as_ggplot(gt) +
  draw_plot_label(label = letters[1:6],# size = 25,
                  x = c(0, 1/4.5, 2/4.5, 3/4.5, 0, 0),
                  y = c(1, 1, 1, 1, 3.4/4.7, 1.2/4.7))

ggsave2("data_summary_visualization_final.png", device = "png",
        plot = final_plot, width = 9.5, height = 10, units = "in", dpi = 300)








