data_filename_list <- c('mouse_retina_drop-seq.rds', 'mouse_sie_10x.rds',
                        'human_blood.rds')
cell_type_sel_list <- matrix(c('bipolar', 'cone', 'rod',
                               'Stem', 'Goblet', 'Tuft',
                               'DC1', 'DC2', 'Mono2'),
                             byrow = FALSE, nrow = 3)
protocol_list <- c('drop-seq', '10x', 'SMART-Seq2')

data_filename_list <- c('pbmc2_hca_drop-seq.rds', 'pbmc2_hca_10x.rds',
                        'pbmc2_hca_SMART-Seq2.rds')
# cell_type_sel <- c('B cell', 'CD14+ monocyte',
#                    'Natural killer cell', 'CD4+ T cell',
#                    'Cytotoxic T cell')
cell_type_sel_list <- matrix(rep(c('B cell', 'CD14+ monocyte',
                                   'Natural killer cell', 'CD4+ T cell',
                                   'Cytotoxic T cell'), length(data_filename_list)),
                             byrow = FALSE, ncol = length(data_filename_list))
cell_type_sel_list
protocol_list <- c('drop-seq', '10x', 'SMART-Seq2')

data_summary_list <- list()
gene_sel_dat_list <- list()

for(iter1 in 1:length(data_filename_list)){
  data_filename <- data_filename_list[iter1]
  data_mat_original <- readRDS(data_filename)
  cell_type_name <- colnames(data_mat_original)
  
  cell_type_name <- colnames(data_mat_original)
  for(iter2 in 1:nrow(cell_type_sel_list)){
    cell_type_sel <- cell_type_sel_list[iter2, iter1]
    output_suffix <- paste0(protocol_list[iter1], '_', cell_type_sel)
    print(cell_type_sel)
    
    set.seed(2)
    data_mat <- data_mat_original[, cell_type_name == cell_type_sel] # 'Stem'
    
    # temp <- get_data_single_cell_type(data_filename, cell_type_sel)
    # data_mat <- cbind(temp$testcount, temp$realcount)
    
    min_value <- 1e-5
    cell_libsize <- apply(data_mat, 2, sum)
    n_gene_detected <- apply(data_mat, 2, function(x){sum(x>min_value)})
    n_cell <- ncol(data_mat)
    
    copula_model <- readRDS(file = paste0('copula_model_', output_suffix, '.rds'))
    copula_model <- copula_model[[1]]
    n_ZI1 <- sum(copula_model$marginal_param1[, 1] > min_value)
    n_non_ZI1 <- sum(copula_model$marginal_param1[, 1] <= min_value)
    n_ZI2 <- sum(copula_model$marginal_param2[, 1] > min_value)
    n_non_ZI2 <- sum(copula_model$marginal_param2[, 1] <= min_value)
    n_nofit <- length(copula_model$gene_sel3)
    n_fit <- length(copula_model$gene_sel1) + length(copula_model$gene_sel2)
    cat('# gene fitted =', nrow(copula_model$marginal_param1) +
          nrow(copula_model$marginal_param2), '\n')
    cat('# ZI =', n_ZI1 + n_ZI2, '\n')
    cat('# non-ZI =', n_non_ZI1 + n_non_ZI2, '\n')
    
    data_summary_list[[output_suffix]] <- list(cell_libsize = cell_libsize,
                                               n_gene_detected = n_gene_detected,
                                               n_cell = n_cell,
                                               n_ZI = n_ZI1+n_ZI2,
                                               n_non_ZI = n_non_ZI1+n_non_ZI2,
                                               n_nofit = n_nofit,
                                               n_fit = n_fit)
  }
}

saveRDS(data_summary_list, 'data_summary_list.rds')


data_mat_original_list <- lapply(data_filename_list, readRDS)
min_value <- 1e-5 # the value to represent 0.0

for(iter2 in 1){
  cell_type_sel <- cell_type_sel_list[iter2, 1]
  print(cell_type_sel)
  
  data_mat_list <- lapply(data_mat_original_list, function(data_mat_original){
    data_mat_original[, colnames(data_mat_original) == cell_type_sel]
  })
  names(data_mat_list) <- protocol_list
  
  data_param <- lapply(1:length(protocol_list), function(iter1){
    output_suffix <- paste0(protocol_list[iter1], '_', cell_type_sel)
    copula_model <- readRDS(file = paste0('copula_model_', output_suffix, '.rds'))
    list(copula_model = copula_model[[1]])
  })
  
  n_gene <- length(data_param[[1]]$copula$gene_sel1) + length(data_param[[1]]$copula$gene_sel2) +
    length(data_param[[1]]$copula$gene_sel3)
  
  param_mat_list <- lapply(1:length(protocol_list), function(iter1){
    param_mat <- matrix(0.0, nrow = n_gene, ncol = 3)
    param_mat[, 2] <- Inf
    param_mat[data_param[[iter1]]$copula_model$gene_sel1, 1:3] <-
      data_param[[iter1]]$copula_model$marginal_param1
    param_mat[data_param[[iter1]]$copula_model$gene_sel2, 1:3] <-
      data_param[[iter1]]$copula_model$marginal_param2
    param_mat
  })
  names(param_mat_list) <- protocol_list
  
  gene_name_sel <- c('EEF1A1', 'ACTB', 'CD79B', 'LCP1')
  print(gene_name_sel)
  
  gene_sel_dat_list <- lapply(protocol_list, function(protocol){
    data_mat <- data_mat_list[[protocol]]
    param_mat <- param_mat_list[[protocol]]
    
    row_sel <- which(rownames(data_mat) %in% gene_name_sel)
    scatter_dat <- data_mat[row_sel, ]
    param_dat <- param_mat[row_sel, ]
    # rownames(scatter_dat) <- gene_name_sel
    rownames(param_dat) <- rownames(scatter_dat)
    
    cat(rownames(scatter_dat), '\n')
    cat(rownames(param_dat), '\n\n')
    
    list(scatter_dat = scatter_dat, param_dat = param_dat)
  })
  names(gene_sel_dat_list) <- paste0(protocol_list, '_', cell_type_sel)
  saveRDS(gene_sel_dat_list, paste0('gene_sel_dat_list_', cell_type_sel, '.rds'))
}











